import math
from adap_gl import AdapGL as agl
from calcroot import Newton
from Calc_z_NormalDistri import calc_z

def sqr( x ):
    return x * x

class C_t_tan_kernel:
    def __init__(self, df):
        self.df = df

    def v(self, theta):
        t = math.tan(theta)
        v = pow(1.0 / (1.0 + sqr(t) / self.df), (self.df + 1.0) / 2.0)
        return v / sqr(math.cos(theta))
                
class C_t_tan_density:
    def __init__(self, df):
        self.df = df
        f_kernel = C_t_tan_kernel(self.df)
        gl = agl()
        v = gl.integral( -math.pi/2.0, math.pi/2.0, f_kernel)
        self.c = 1.0 / v

    def v(self, theta):
        t = math.tan(theta)
        v = pow(1.0 / (1.0 + sqr(t) / self.df), (self.df + 1.0) / 2.0)
        return self.c * v / sqr(math.cos(theta))
        
class C_cum_t:
    def __init__(self, df):
        self.df = df
        self.t_tan = C_t_tan_density(self.df)
        self.gl = agl()

    def v( self, t ):
        t_cum = self.gl.integral(-math.pi/2.0, math.atan(t), self.t_tan)
        return t_cum

class C_dcum_t:
    def __init__(self, df):
        self.df = df
        f_kernel = C_t_tan_kernel(self.df)
        gl = agl()
        v = gl.integral( -math.pi/2.0, math.pi/2.0, f_kernel)
        self.c = 1.0 / v

    def v(self, t):
        tv = self.c * pow(1.0 / (1.0 + sqr(t) / self.df), (self.df + 1.0) / 2.0)
        return tv

def calc_p_to_t( p, df ):
    cum_t = C_cum_t( df )
    Lb = 0.0
    while (cum_t.v(Lb) >= p):
        Lb -= 1.0
    Ub = Lb + 1.0
    while (cum_t.v(Ub) <= p):
        Ub += 1.0

    dcum_t = C_dcum_t( df )
    t = Newton( cum_t, dcum_t, p, Lb, Ub )
    return t


while True:
    try:
        s = input("df = ")
        df = float(s)
        s = input("p = ")
        p = float(s)
        t = calc_p_to_t( p, df )
        print("t = %f     p = %f   df = %f"%(t, p, df))
    except:
        break

