import math
from adap_gl import AdapGL as agl

def sqr( x ):
    return x * x

class C_t_tan_kernel:
    def __init__(self, df):
        self.df = df

    def v(self, theta):
        t = math.tan(theta)
        v = pow(1.0 / (1.0 + sqr(t) / self.df), (self.df + 1.0) / 2.0)
        return v / sqr(math.cos(theta))
               
class C_t_tan_density:
    def __init__(self, df):
        self.df = df
        f_kernel = C_t_tan_kernel(self.df)
        gl = agl()
        v = gl.integral( -math.pi/2.0, math.pi/2.0, f_kernel)
        print("v = %f"%v)
        self.c = 1.0 / v

    def v(self, theta):
        t = math.tan(theta)
        v = pow(1.0 / (1.0 + sqr(t) / self.df), (self.df + 1.0) / 2.0)
        return self.c * v / sqr(math.cos(theta))
        
class C_cum_t:
    def __init__(self, df):
        self.df = df
        self.t_tan = C_t_tan_density(self.df)
        self.gl = agl()

    def v( self, t ):
        t_cum = self.gl.integral(-math.pi/2.0, math.atan(t), self.t_tan)
        return t_cum


class NormalDistri:
    def __init__(self):
        self.c = 1.0 / math.sqrt(2.0 * math.pi)

    def v(self, z):
        return self.c * math.exp(-0.5 * z * z)

n_func = NormalDistri()

gl = agl()

def cum_normal( z ):
    if z > 0.0:
        return 0.5 + gl.integral(0.0, z, n_func)
    elif z < 0.0:
        return 0.5 - gl.integral(z, 0.0, n_func)
    else:
        return 0.5


cum_t = C_cum_t(1000)
t = -10.0
while True:
    p = cum_t.v(t)
    pn = cum_normal(t)
    print("t = %f    p = %f    pn = %f     p - pn = %f"%(t, p, pn, p - pn))
    t += 0.5
    if t > 10.1:
        break



